:root {
    --md-primary-fg-color:             #001425;
    --md-primary-fg-color--light:      #308AAD;
    --md-primary-fg-color--dark:       #001425;
    --md-accent-fg-color:              #308AAD;
    --md-accent-fg-color--transparent: #308AAD;
    --md-typeset-a-color:              #308AAD;
  }

  p a {
    text-decoration: underline;
  }
